/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


#ifndef DIMPRO_NODIMENSIONSCALAR_HPP_
#define DIMPRO_NODIMENSIONSCALAR_HPP_

#include "scalar.hpp"

namespace NSDimensionPromise
{

/**
 * NoDimensionScalarの次元定義.
 */
template <typename T>
struct NoDimensionScalarTraits
{
    typedef ScalarNumber<T, 0, 0, 0, 0, 0, 0, 0> ScalarType;
};

/**
 * NoDimensionScalarを表すクラス.
 * 組み込み型から無次元スカラーへの暗黙的な変換をサポート
 */
class NoDimensionScalar : public NoDimensionScalarTraits<double>::ScalarType
{
public:
    /**
     * 無次元スカラー変数を生成する.
     * @param x 変数の初期値
     */
    NoDimensionScalar(const double& x = double())
        : NoDimensionScalarTraits<double>::ScalarType(x)
    {  }

    /**
     * 無次元スカラー変数をコピーして生成する.
     * @param rhs コピー元の変数
     */
    NoDimensionScalar(const NoDimensionScalarTraits<double>::ScalarType& rhs)
        : NoDimensionScalarTraits<double>::ScalarType(rhs)
    {  }

    /**
      * x単位の値を得る.
      * @return x
      */
    const double& Value() const
    {
        return this->X();
    }
};

}

#endif /* DIMPRO_NODIMENSIONSCALAR_HPP_ */
