/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * SourceMaker.cpp
 *
 *  Created on: 2010/11/22
 *      Author: HARK Development Team
 */

#include <cmath>

#include <BufferedNode.h>
#include <Buffer.h>
#include <Vector.h>
#include <Source.h>

#include <DimensionPromise/dimpro.hpp>

using namespace std;
using namespace FD;
using namespace NSDimensionPromise;

class SourceMaker;

DECLARE_NODE(SourceMaker);
/*Node
 *
 * @name SourceMaker
 * @category HARK:BINAURAL
 * @description Make a struct "Source".
 *
 * @output_name OUTPUT
 * @output_type Vector<ObjectRef>
 * @output_description source for testing.
 *
 * @parameter_name DIRECTIONS
 * @parameter_type object
 * @parameter_value <Vector<float> 0.0>
 * @parameter_description Separation directions in degree.
 *
 END*/

class SourceMaker : public BufferedNode
{
public:
    SourceMaker(string nodeName, ParameterSet params)
        : BufferedNode(nodeName, params)
    {
        outputID = addOutput("OUTPUT");

        directions = object_cast<Vector<float> >(parameters.get("DIRECTIONS"));
    }

    virtual ~SourceMaker()
    { }

    void calculate(int output_id, int count, Buffer &out)
    {
        // 出力変数
        RCPtr<Vector<ObjectRef> > sourcesPtr(Vector<ObjectRef>::alloc(0));
        Vector<ObjectRef>& sources = *sourcesPtr;

        RCPtr<Source> source;

        for (size_t i = 0; i < directions.size(); ++i)
        {
            source = RCPtr<Source>(new Source());
            source->id = i;
            source->x[0] = cos(Angle::FromDegree(directions.at(i)).ToRadian());
            source->x[1] = sin(Angle::FromDegree(directions.at(i)).ToRadian());
            source->x[2] = 0;
            sources.push_back(source);
        }

        (*outputs[outputID].buffer)[count] = sourcesPtr;
    }

private:
    int outputID;
    Vector<float> directions;
};

